# -*- coding: utf-8 -*-

# +-----------------------------------------------------+
# | Commande d'une LED sur le port GPIO du Raspberry Pi |
# +-----------------------------------------------------+

# python 2.7
# (C) Fabrice Sincère

# nécessite les droits superutilisateurs avec Raspian Wheezy

# Bibliographie :
# https://sourceforge.net/p/raspberry-gpio-python/wiki/BasicUsage
# https://sourceforge.net/p/raspberry-gpio-python/wiki/Outputs/
# https://sourceforge.net/p/raspberry-gpio-python/wiki/Inputs/

version = "1.0.2"

import sys

from Tkinter import *
from tkMessageBox import *

try:
    import RPi.GPIO as GPIO
except:
    print sys.exc_info()[1]
    sys.exit(1)

GPIO.setmode(GPIO.BOARD) # board numbering system
GPIO.setwarnings(False) # disable warnings

def setoutput():
    """ choix de la broche de sortie
    
Connecteur GPIO 40 broches

+--+--+--+-----+--+
| 2| 4| 6|.....|40|
+--+--+--+-----+--+
| 1| 3| 5|.....|39|
+--+--+--+-----+--+
"""

    global channel, configuration

    if configuration == True:
        GPIO.cleanup(channel)

    configuration = False
    # désactivation des boutons ON/OFF
    bouton_on["state"] = DISABLED
    bouton_off["state"] = DISABLED
    bouton_off["bg"] = "#d9d9d9"
    bouton_on["bg"] = "#d9d9d9"
    
    try:
        channel = int(numero_broche.get())
    except:
        channel = defaultchannel
    
    numero_broche.set(channel)
    
    # shows the function of a GPIO channel
    try:
        # nécessite les droits superutilisateurs avec Raspian Wheezy
        func = GPIO.gpio_function(channel)
    except:
        affichage = sys.exc_info()[1]
        showerror('Résultat', affichage)
        return
    
    if func not in [GPIO.IN, GPIO.OUT]:
        # GPIO.SPI, GPIO.I2C, GPIO.HARD_PWM, GPIO.SERIAL, GPIO.UNKNOWN
        funcs = {0:"OUT", 1:"IN", 42:"I2C", 41:"SPI", 40:"SERIAL", -1:"UNKNOWN", 43:"HARD_PWM"}
        affichage = "Broche non utilisable ({})".format(funcs[func])
        showerror('Résultat', affichage)
        return


    GPIO.setup(channel, GPIO.OUT) # set up a channel as an output
    configuration = True
        
    affichage =  "Configuration de la broche n°{} en sortie [OK]".format(channel)
    showinfo('Résultat', affichage)

    # activation des boutons ON/OFF
    bouton_on["state"] = NORMAL
    bouton_off["state"] = NORMAL

    # lecture niveau et coloration des boutons ON/OFF
    if GPIO.input(channel) == True:
        bouton_off["bg"] = "#d9d9d9"
        bouton_on["bg"] = "green"
        bouton_on["activebackground"] = "#00dd00"
    else:
        bouton_off["bg"] = "red"
        bouton_on["bg"] = "#d9d9d9"
        bouton_off["activebackground"] = "#dd0000"

def allumer():
    """ allume la LED """

    if configuration == False:
        return

    # GPIO.output(channel, not GPIO.input(channel)) # autre technique
    
    if GPIO.input(channel) == False:
        # sortie mis à 1
        GPIO.output(channel, GPIO.HIGH)
        # relecture pour vérification
        if GPIO.input(channel) == True:
            bouton_on["bg"] = "green"
            bouton_on["activebackground"] = "#00dd00"
            bouton_off["bg"] = "#d9d9d9"
        else:
            showerror('', 'Erreur inattendue')
    
    
def eteindre():
    """ éteindre la led """

    if configuration == False:
        return

    # sortie mis à 0
    GPIO.output(channel, GPIO.LOW)
    # relecture pour vérification
    if GPIO.input(channel) == False:
        bouton_off["bg"] = "red"
        bouton_off["activebackground"] = "#dd0000"
        bouton_on["bg"] = "#d9d9d9"
    else:
        showerror('', 'Erreur inattendue')

def quitter():

    if configuration == True:
        GPIO.cleanup(channel)
        
    Mafenetre.destroy()
    
    
# Création de la fenêtre principale (main window)
Mafenetre = Tk()
Mafenetre.title('LED')
Mafenetre.resizable(width=False, height=False)

police = ('Arial', 14)

# Création d'un widget Label
label_broche = Label(Mafenetre, text='Broche n°', fg='blue')
label_broche['font'] = police
# Positionnement du widget avec la méthode grid()
label_broche.grid(row=0, column=0, padx=5, pady=5)

# Création d'un widget Entry (champ de saisie)
numero_broche = StringVar() # variable associée au champ


defaultchannel = 16
numero_broche.set(defaultchannel)

# variable globale
channel = defaultchannel # position sur le connecteur 40 broches
configuration = False # sortie non configurée

Champ = Entry(Mafenetre, textvariable=numero_broche, width=5, bg='bisque', fg='maroon')
Champ['font'] = police
Champ.focus_set()
Champ.grid(row=1, column=0, padx=5, pady=5)

# Création d'un widget Button (bouton Valider)
bouton_valider = Button(Mafenetre, text='Valider', command=setoutput, font=police)
bouton_valider.grid(row=0, column=1, rowspan=2, sticky=W+E+N+S, padx=5, pady=5)

# Création d'un widget Button (bouton ON)
bouton_on = Button(Mafenetre, text='ON', command=allumer, state=DISABLED, font=police)
bouton_on.grid(row=2, column=0, sticky=W+E, padx=5, pady=10)

# Création d'un widget Button (bouton OFF)
bouton_off = Button(Mafenetre, text='OFF', command=eteindre, state=DISABLED, font=police)
bouton_off.grid(row=2, column=1, sticky=W+E, padx=5, pady=10)

# Création d'un widget Button (bouton Quitter)
boutonquitter = Button(Mafenetre, text = 'Quitter', command=quitter , font=police)
boutonquitter.grid(row=3, column=0, columnspan=2, padx=5, pady=5)

# Lancement du gestionnaire d'événements
Mafenetre.mainloop()
