# -*- coding: utf-8 -*-

# +----------------------------------------------------------------------+
# | Configuration en entrée et lecture sur le port GPIO du Raspberry Pi  |
# +----------------------------------------------------------------------+

# python 2.7
# (C) Fabrice Sincère

# nécessite les droits superutilisateurs avec Raspian Wheezy
# fonctionne depuis IDLE

# Bibliographie :
# https://sourceforge.net/p/raspberry-gpio-python/wiki/BasicUsage
# https://sourceforge.net/p/raspberry-gpio-python/wiki/Outputs/
# https://sourceforge.net/p/raspberry-gpio-python/wiki/Inputs/

version = "1.0.0"

import sys

from Tkinter import *
from tkMessageBox import *

try:
    import RPi.GPIO as GPIO
except:
    print sys.exc_info()[1]
    sys.exit(1)

GPIO.setmode(GPIO.BOARD) # board numbering system
GPIO.setwarnings(False) # disable warnings


def setinput():
    """ choix de la broche d'entrée

Connecteur GPIO 40 broches

+--+--+--+-----+--+
| 2| 4| 6|.....|40|
+--+--+--+-----+--+
| 1| 3| 5|.....|39|
+--+--+--+-----+--+
"""

    global channel, configuration

    if configuration == True:
        GPIO.remove_event_detect(channel)
        GPIO.cleanup(channel)

    configuration = False
    # label_niveau : paramètres par défaut
    label_niveau["text"] = "X"
    label_niveau["bg"] = "white"


    try:
        channel = int(numero_broche.get())
    except:
        channel = defaultchannel

    numero_broche.set(channel)

    # shows the function of a GPIO channel
    try:
        # nécessite les droits superutilisateurs avec Raspian Wheezy
        func = GPIO.gpio_function(channel)
    except:
        affichage = sys.exc_info()[1]
        showerror('Résultat', affichage)
        return

    if func not in [GPIO.IN, GPIO.OUT]:
        # GPIO.SPI, GPIO.I2C, GPIO.HARD_PWM, GPIO.SERIAL, GPIO.UNKNOWN
        funcs = {0:"OUT", 1:"IN", 42:"I2C", 41:"SPI", 40:"SERIAL", -1:"UNKNOWN", 43:"HARD_PWM"}
        affichage = "Broche non disponible ({})".format(funcs[func])
        showerror('Résultat', affichage)
        return

    # configuration en entrée
    GPIO.setup(channel, GPIO.IN, pull_up_down=GPIO.PUD_UP)

    configuration = True

    affichage = """Configuration de la broche n°{} en entrée avec résistance de pull-up activée [OK]
Veuillez brancher un interrupteur ou un bouton poussoir entre la broche n°{} et GND.""".format(channel, channel)

    showinfo('Résultat', affichage)

    # lecture niveau et mise à jour label_niveau
    if GPIO.input(channel) == True:
        label_niveau["text"] = "OFF"
        label_niveau["bg"] = "red"
    else:
        label_niveau["text"] = "ON"
        label_niveau["bg"] = "green"

    # edge detection & antirebonds
    # threaded callback (mécanisme disponible uniquement sur des entrées)
    GPIO.add_event_detect(channel, GPIO.BOTH, callback=affichageniveau, bouncetime=10)


def affichageniveau(channel):
    """ fonction appelée quand le niveau d'entrée change
    add_event_detect : callback function """

    if GPIO.input(channel) == True:
        # niveau 1
        # interrupteur ouvert
        label_niveau["text"] = "OFF"
        label_niveau["bg"] = "red"
    else:
        # niveau 0
        # interrupteur fermé
        label_niveau["text"] = "ON"
        label_niveau["bg"] = "green"

def quitter():

    if configuration == True:
        GPIO.remove_event_detect(channel)
        GPIO.cleanup(channel)

    Mafenetre.destroy()



# Création de la fenêtre principale (main window)
Mafenetre = Tk()
Mafenetre.title('Interrupteur')
Mafenetre['bg'] = "#eeeeee"
Mafenetre.resizable(width=False, height=False)

police = ('Arial', 14)

# Création d'un widget Label
label_broche = Label(Mafenetre, text='Broche n°', bg="#eeeeee", fg='blue')
label_broche['font'] = police
# Positionnement du widget avec la méthode grid()
label_broche.grid(row=0, column=0, padx=5, pady=5)

# Création d'un widget Entry (champ de saisie)
numero_broche = StringVar() # variable associée au champ


defaultchannel = 22 # position sur le connecteur 40 broches
numero_broche.set(defaultchannel)

# variables globales
channel = defaultchannel 
configuration = False # sortie non configurée

Champ = Entry(Mafenetre, textvariable=numero_broche, width=5, bg='bisque', fg='maroon')
Champ['font'] = police
Champ.focus_set()
Champ.grid(row=1, column=0, padx=5, pady=5)

# Création d'un widget Button (bouton Valider)
bouton_valider = Button(Mafenetre, text='Valider', command=setinput, font=police)
bouton_valider.grid(row=0, column=1, rowspan=2, sticky=W+E+N+S, padx=5, pady=5)

# Création d'un widget Label
label_niveau = Label(Mafenetre, text='X', bg='white')
label_niveau['font'] = police
# Positionnement du widget avec la méthode grid()
label_niveau.grid(row=2, column=0, columnspan=2, sticky=W+E+N+S, padx=5, pady=5)

# Création d'un widget Button (bouton Quitter)
boutonquitter = Button(Mafenetre, text = 'Quitter', command=quitter , font=police)
boutonquitter.grid(row=3, column=0, columnspan=2, padx=5, pady=5)

# Lancement du gestionnaire d'événements
Mafenetre.mainloop()




