# -*- coding: utf-8 -*-

# python 2 ou 3
# version 0.0.1

# graphe déroulant dans le temps
# du niveau logique d'une entrée sur le port GPIO

# (C) Fabrice Sincère

# Bibliographie :
# http://matplotlib.org/1.4.0/examples/animation/animate_decay.html

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import time
import sys

version = "0.0.3"

print """
+----------------------------------------------------------------------+
| Configuration en entrée et lecture sur le port GPIO du Raspberry Pi  |
| Graphe déroulant                                                     |
+----------------------------------------------------------------------+

Version {}
""".format(version)

try:
    import RPi.GPIO as GPIO
except:
    print sys.exc_info()[1]
    sys.exit(1)


# choix de la broche

GPIO.setmode(GPIO.BOARD) # board numbering system

print """
Connecteur GPIO 40 broches

+--+--+--+-----+--+
| 2| 4| 6|.....|40|
+--+--+--+-----+--+
| 1| 3| 5|.....|39|
+--+--+--+-----+--+
"""

defaultchannel = 22 # position sur le connecteur 40 broches

try:
    channel = int(raw_input("N° de broche du GPIO ? "))
except:
    channel = defaultchannel

# shows the function of a GPIO channel
try:
    func = GPIO.gpio_function(channel)
    # nécessite les droits superutilisateurs avec Raspian Wheezy
except:
    print sys.exc_info()[1]
    sys.exit(2)

funcs = {0:"OUT", 1:"IN", 42:"I2C", 41:"SPI", 40:"SERIAL", -1:"UNKNOWN", 43:"HARD_PWM"}

if func not in [GPIO.IN, GPIO.OUT]:
    # GPIO.SPI, GPIO.I2C, GPIO.HARD_PWM, GPIO.SERIAL, GPIO.UNKNOWN    
    print "Broche non disponible ({})".format(funcs[func])
    sys.exit(3)

# setup 

GPIO.setwarnings(False) # disable warnings
# "this channel is already in use, continuing anyway"

GPIO.setup(channel, GPIO.IN, pull_up_down=GPIO.PUD_UP) # configuration en entrée

print """Configuration de la broche n°{} en entrée
avec résistance de pull-up activée [OK]

Veuillez brancher un interrupteur ou un bouton poussoir entre la broche n°{} et GND
""".format(channel, channel)

DELTAT = 20.0 # en secondes

fig, ax = plt.subplots()
line, = ax.plot([], [], lw=2)
ax.set_ylim(-0.1, 1.1)
ax.set_xlim(-DELTAT, 0.0)
ax.grid()
xdata, ydata = [], []
plt.xlabel("Temps [s]")
plt.ylabel("Niveau")
plt.title('Broche ' + str(channel))

def run(data):
    
    # update the data
    t, y = data
    xdata.append(t)
    ydata.append(y)
    xmin, xmax = ax.get_xlim()

    if t > xmax:
        #refresh graph
        #ax.set_xlim(xmin + DELTAT, xmax + DELTAT)  # type 1
        ax.set_xlim(xmax - DELTAT, t)              # type 2    
        
    line.set_data(xdata, ydata)
    return


def data_gen():
    # lecture niveau initial (0/1)
    VAL = int(GPIO.input(channel))
    VAL0 = VAL # mémorisation
    
    tinit = time.time() # timestamp initial

    while True:
        t = time.time() - tinit
        # lecture niveau 0/1
        VAL = int(GPIO.input(channel)) # lecture niveau
        if VAL != VAL0:
            # toggle
            yield t, VAL0
        yield t, VAL    
        VAL0 = VAL

ani = animation.FuncAnimation(fig, run, data_gen, interval=10)
plt.show()

raw_input("Touche Enter pour quitter.")

GPIO.cleanup(channel)
print "Bye !"
